package com.persagy.dmp.admin.service;

import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.admin.entity.QTransObjectInfo;
import com.persagy.dmp.admin.entity.TransObjectInfo;
import com.persagy.dmp.admin.model.DownloadModel;
import com.persagy.dmp.admin.repository.TransObjectInfoRepository;
import com.persagy.dmp.common.excel.ExcelUtils;
import com.persagy.dmp.common.excel.SheetReadInfo;
import com.persagy.dmp.common.excel.SheetWriteInfo;
import org.apache.commons.collections4.MapUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.transaction.Transactional;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @author: yaoll
 * @date: 2021-01-04
 * @verison: 1.0
 */
@Service
public class TransObjectInfoService {

    @Autowired
    private CriteriaUtils criteriaUtils;

    @Autowired
    private TransObjectInfoRepository objectInfoRepository;

    @Autowired
    private DmpTransObjectService transObjectService;

    public PagedResponse<TransObjectInfo> query(JacksonCriteria criteria) {
        return criteriaUtils.query(QTransObjectInfo.transObjectInfo, null, criteria);
    }

    @Transactional
    public MapResponse deleteByProjectId(String projectId) {
        MapResponse resp = new MapResponse();
        if (projectId != null) {
            objectInfoRepository.deleteByProjectId(projectId);
        }
        return resp;
    }

    public DownloadModel download(String projectId) {
        DownloadModel result = new DownloadModel();
        result.setName("异常数据-" + projectId + ".xlsx");
        List<SheetWriteInfo> list = new ArrayList<>(1);
        SheetWriteInfo<TransObjectInfo> info = new SheetWriteInfo<>();
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.setWithoutCount(true);
        criteria.setOrders(Arrays.asList(new JacksonCriteria.Order("id", true)));
        criteria.add("projectId", projectId);
        criteria.add("type", "error");
        info.setData(query(criteria).getData());
        info.setSheetName("异常数据");
        info.addColumn("项目ID", "projectId", "string", 30, HorizontalAlignment.CENTER);
        info.addColumn("对象类型", "classCode", "string", 30, HorizontalAlignment.CENTER);
        info.addColumn("对象ID", "objectId", "string", 80, HorizontalAlignment.CENTER);
        info.addColumn("信息点编码", "infoCode", "string", 30, HorizontalAlignment.CENTER);
        info.addColumn("信息点值", "infoCodeValue", "string", 130, HorizontalAlignment.CENTER);
        info.addColumn("类型", "type", "string", 30, HorizontalAlignment.CENTER);
        info.addColumn("异常信息", "message", "string", 200, HorizontalAlignment.LEFT);
        info.setDataHandler((data, prop) -> {
            switch (prop) {
                case "projectId":
                    return data.getProjectId();
                case "type":
                    return data.getType();
                case "classCode":
                    return data.getClassCode();
                case "objectId":
                    return data.getObjectId();
                case "infoCode":
                    return data.getInfoCode();
                case "infoCodeValue":
                    return data.getInfoCodeValue();
                case "message":
                    return data.getMessage();
            }
            return null;
        });
        list.add(info);
        Workbook workbook = ExcelUtils.createExcel(list);
        result.setWorkbook(workbook);
        return result;
    }

    public MapResponse upload(String groupCode, String projectId, MultipartFile file) {
        List<TransObjectInfo> transObjectInfoList = null;
        try {
            transObjectInfoList = CompareDataExcelRead(file);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return transObjectService.againTrans(groupCode, projectId, transObjectInfoList);
    }

    public List<TransObjectInfo> CompareDataExcelRead(MultipartFile file) throws IOException {
        SheetReadInfo info = new SheetReadInfo();
        info.setStartRow(1);
        info.add(0, "projectId", "string");
        info.add(1, "classCode", "string");
        info.add(2, "objectId", "string");
        info.add(3, "infoCode", "string");
        info.add(4, "infoCodeValue", "string");
        info.add(5, "type", "string");
        info.add(6, "message", "string");

        List<Map<String, Object>> result = ExcelUtils.read(file.getInputStream(), info);

        List<TransObjectInfo> transObjectInfoList = new ArrayList<>();
        for (Map<String, Object> map : result) {
            String projectId = MapUtils.getString(map, "projectId");
            String classCode = MapUtils.getString(map, "classCode");
            String objectId = MapUtils.getString(map, "objectId");
            String infoCode = MapUtils.getString(map, "infoCode");
            String infoCodeValue = MapUtils.getString(map, "infoCodeValue");
            String type = MapUtils.getString(map, "type");
            String message = MapUtils.getString(map, "message");
            if (classCode != null) {
                System.out.println();
            }

            TransObjectInfo transObjectInfo = new TransObjectInfo();
            transObjectInfo.setProjectId(projectId);
            transObjectInfo.setClassCode(classCode);
            transObjectInfo.setObjectId(objectId);
            transObjectInfo.setInfoCode(infoCode);
            transObjectInfo.setInfoCodeValue(infoCodeValue);
            transObjectInfo.setType(type);
            transObjectInfo.setMessage(message);
            transObjectInfoList.add(transObjectInfo);
        }
        return transObjectInfoList;
    }
}
