package com.persagy.dmp.report;

import com.persagy.common.criteria.JpaUtils;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.report.common.criteria.CriteriaUtils;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.EnableRabbit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.cloud.client.SpringCloudApplication;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.scheduling.annotation.EnableScheduling;

import javax.persistence.EntityManager;

/**
 * @author: fengyanjie
 * @date: 2020-11-2
 * @verison: 1.0
 */
@Slf4j
@EnableRabbit
@SpringCloudApplication
@EnableFeignClients
@EnableScheduling
public class ReportApp {

    public static void main(String[] args) {
        SpringApplication.run(ReportApp.class, args);
    }

    @Bean
    public JPAQueryFactory jpaQueryFactory(EntityManager entityManager) {
        return new JPAQueryFactory(entityManager);
    }

    @Bean
    public CriteriaUtils criteriaUtils(@Autowired JPAQueryFactory jpaQueryFactory) {
        return new CriteriaUtils(jpaQueryFactory);
    }

    @Bean
    public JpaUtils jpaUtils(@Autowired EntityManager entityManager) {
        return new JpaUtils(entityManager);
    }

    @Bean
    public HttpUtils httpUtils() {
        return new HttpUtils();
    }

    @Bean
    public SpringUtils springUtils() {
        return new SpringUtils();
    }
}