package com.persagy.dmp.report.jms;

import com.persagy.common.json.JacksonMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.Map;

/**
 * @author: fengyanjie
 * @date: 2020-11-10
 * @verison: 1.0
 */
@Slf4j
@Configuration
public class JmsConfig {


    @Value("${dmp.report.exchange}")
    private String exchange;

    @Value("${dmp.report.routingKey}")
    private String reportRoutingKey;

    private String alarmRoutingKey = "routing-dmp-alarm";
    private String rwdRoutingKey = "routing-dmp-rwd";

    @Value("${dmp.report.queue}")
    private String reportQueue;

    @Autowired
    private MessageProcesser messageProcesser;

    @Bean
    public TopicExchange exchange() {
        return new TopicExchange(exchange);
    }

    @Bean
    public Queue queue() {
        return new Queue(reportQueue, true);
    }

    @Bean
    public Binding reportBinding() {
        return BindingBuilder.bind(queue()).to(exchange()).with(reportRoutingKey);
    }

    @Bean
    public Binding alarmBinding() {
        return BindingBuilder.bind(queue()).to(exchange()).with(alarmRoutingKey);
    }

    @Bean
    public Binding rwdBinding() {
        return BindingBuilder.bind(queue()).to(exchange()).with(rwdRoutingKey);
    }

    @RabbitListener(queues = "${dmp.report.queue}")    //监听器监听指定的Queue
    public void processC(Map<String, Object> message) {
        log.info("============================== Receive:" + JacksonMapper.toSimpleJson(message));
        messageProcesser.listen(message);
    }
}
