package com.persagy.dmp.report.jms;

import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.report.enumeration.EnumReportMessageType;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * @author: fengyanjie
 * @date: 2020-11-10
 * @verison: 1.0
 */
@Slf4j
@Component
public class MessageProcesser {

    @Value("${dmp.report.exchange}")
    private String exchange;

    @Value("${dmp.report.routingKey}")
    private String reportRoutingKey;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    public void convertAndSend(DmpMessage message) {
        log.info("向rabbitMQ发送消息，message:" + JacksonMapper.toSimpleJson(message));
        rabbitTemplate.convertAndSend(exchange, reportRoutingKey, message);
    }

    private static final Set<String> beanNames = new HashSet<>();

    static {
        beanNames.add(EnumReportMessageType.REPORT_RWD_OBJECT_CREATE.getValue());
        beanNames.add(EnumReportMessageType.ALARM_RECORD_CREATE.getValue());
        beanNames.add(EnumReportMessageType.RWD_OBJECT_CREATE.getValue());
    }

    public void listen(Map<String, Object> message) {
        String type = "";
        if (null != MapUtils.getString(message, "type")) {
            type = MapUtils.getString(message, "type");
        } else {
            type = MapUtils.getString(message, "beanName");
        }
        if (!beanNames.contains(type)) {
            return;
        }

        String executorName = type + "Executor";

        MessageExecutor executor = null;
        try {
            executor = SpringUtils.getBean(executorName, MessageExecutor.class);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        if (executor == null) {
            log.error("bean for [" + executorName + "] not exists");
        }

        try {
            DmpParameterStorage.ParameterData data = new DmpParameterStorage.ParameterData();
            data.setGroupCode(MapUtils.getString(message, "groupCode"));
            data.setProjectId(MapUtils.getString(message, "userId"));
            data.setAppId(MapUtils.getString(message, "appId"));
            DmpParameterStorage.set(data);

            executor.execute(message);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            DmpParameterStorage.clear();
        }
    }

}
