package com.persagy.dmp.report.service;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.criteria.JpaUtils;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.report.feign.RwdClient;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author fengyanjie
 */
@Slf4j
@Service
public class GroupColumnService {

    @Autowired
    private JpaUtils jpaUtils;

    @Autowired
    JPAQueryFactory queryFactory;

    @Autowired
    private RwdClient rwdClient;

    public List<Object[]> itemCode() {
        List<Object[]> result = new ArrayList<>();
        String sql = "select distinct code from dmp_alarm.alarm_item where valid = 1";
        List<Map<String, Object>> list = jpaUtils.queryList(sql);
        for (Map<String, Object> map : list) {
            result.add(new String[]{MapUtils.getString(map, "code")});
        }
        return result;
    }

    public List<Object[]> category() {
        List<Object[]> result = new ArrayList<>();
        String sql = "select distinct category from dmp_alarm.alarm_item where valid = 1";
        List<Map<String, Object>> list = jpaUtils.queryList(sql);
        for (Map<String, Object> map : list) {
            result.add(new String[]{MapUtils.getString(map, "category")});
        }
        return result;
    }

    public List<Object[]> classCode() {
        PagedResponse<ClassDefModel> classResponse = rwdClient.queryClass(null, JacksonCriteria.newInstance());
        if (!classResponse.success()) {
            return null;
        }
        List<ClassDefModel> data = classResponse.getData();
        List<String> list = data.stream().map(ClassDefModel::getCode).distinct().collect(Collectors.toList());
        List<Object[]> result = new ArrayList<>();
        for (String code : list) {
            result.add(new String[]{code});
        }
        return result;
    }

    public List<Object[]> classCodeObjType() {
        PagedResponse<ClassDefModel> classResponse = rwdClient.queryClass(null, JacksonCriteria.newInstance());
        if (!classResponse.success()) {
            return null;
        }
        List<ClassDefModel> data = classResponse.getData();
        List<ClassDefModel> list = data.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(item -> item.getCode() + "#" + item.getObjType()))), ArrayList::new));
        List<Object[]> result = new ArrayList<>();
        for (ClassDefModel model : list) {
            result.add(new String[]{model.getCode(), model.getObjType().name()});
        }
        return result;
    }
}
